// src/index.ts
import {
  queryOptions as defineQueryOptions,
  skipToken,
  useInfiniteQuery,
  useMutation,
  useQueries,
  useQuery
} from "@tanstack/vue-query";
import { createTRPCUntypedClient } from "@trpc/client";
import { createTRPCFlatProxy } from "@trpc/server";
import { createRecursiveProxy } from "@trpc/server/unstable-core-do-not-import";
import { toRef, toRefs } from "@vueuse/core";
import { computed, isReactive, onScopeDispose, shallowRef, toValue, watch } from "vue";
function getQueryKey(path, input, type) {
  const splitPath = path.flatMap((part) => part.split("."));
  if (input === void 0 && !type) {
    return splitPath.length > 0 ? [splitPath] : [];
  }
  return [
    splitPath,
    {
      ...input !== void 0 && input !== skipToken && { input },
      ...type && { type }
    }
  ];
}
function maybeToRefs(obj) {
  return isReactive(obj) ? toRefs(obj) : toRefs(toRef(obj));
}
function createVueQueryProxyDecoration(name, trpc, queryClient) {
  return createRecursiveProxy(({ args, path: _path }) => {
    const path = [name, ..._path];
    const prop = path.pop();
    const joinedPath = path.join(".");
    const [firstArg, ...rest] = args;
    const opts = rest[0] || {};
    if (prop === "_def") {
      return { path };
    }
    if (prop === "query") {
      return trpc.query(joinedPath, firstArg, opts);
    }
    function createQuery(_input, { trpcOptions, queryOptions }, { type = "query" } = {}) {
      const queryFn = computed(
        () => toValue(_input) === skipToken ? skipToken : async ({ signal }) => {
          const input = toValue(_input);
          const output = await trpc.query(joinedPath, input, {
            signal,
            ...trpcOptions
          });
          if (type === "queries") return { output, input };
          return output;
        }
      );
      return defineQueryOptions({
        queryKey: computed(() => getQueryKey(path, toValue(_input), type)),
        queryFn,
        ...maybeToRefs(queryOptions)
      });
    }
    if (prop === "useQuery") {
      const { trpc: trpcOptions, ...queryOptions } = opts;
      const input = firstArg;
      return useQuery(createQuery(input, { trpcOptions, queryOptions }));
    }
    if (prop === "useQueries") {
      const { trpc: trpcOptions, combine, shallow, ...queryOptions } = opts;
      const inputs = firstArg;
      return useQueries({
        queries: computed(
          () => toValue(inputs).map(
            (i) => createQuery(i, { trpcOptions, queryOptions }, { type: "queries" })
          )
        ),
        combine,
        ...maybeToRefs({ shallow })
      });
    }
    if (prop === "invalidate") {
      return queryClient.invalidateQueries({
        queryKey: getQueryKey(path, toValue(firstArg), "query")
      });
    }
    if (prop === "setQueryData") {
      return queryClient.setQueryData(getQueryKey(path, toValue(opts), "query"), firstArg);
    }
    if (prop === "key") {
      return getQueryKey(path, toValue(firstArg), "query");
    }
    if (prop === "mutate") {
      return trpc.mutation(joinedPath, firstArg, opts);
    }
    if (prop === "useMutation") {
      const { trpc: trpcOptions, ...mutationOptions } = firstArg || {};
      return useMutation({
        mutationKey: computed(() => getQueryKey(path, void 0)),
        mutationFn: async (payload) => trpc.mutation(joinedPath, payload, {
          ...trpcOptions
        }),
        ...maybeToRefs(mutationOptions)
      });
    }
    if (prop === "subscribe") {
      return trpc.subscription(joinedPath, firstArg, opts);
    }
    if (prop === "useSubscription") {
      const inputData = toRef(firstArg);
      const subscription = shallowRef();
      watch(
        inputData,
        () => {
          if (inputData.value === skipToken) return;
          subscription.value?.unsubscribe();
          subscription.value = trpc.subscription(joinedPath, inputData.value, {
            ...opts
          });
        },
        { immediate: true }
      );
      onScopeDispose(() => {
        subscription.value?.unsubscribe();
      }, true);
      return subscription.value;
    }
    if (prop === "useInfiniteQuery") {
      const { trpc: trpcOptions, ...queryOptions } = opts;
      return useInfiniteQuery({
        queryKey: computed(() => getQueryKey(path, toValue(firstArg), "infinite")),
        queryFn: async ({ queryKey, pageParam, signal }) => trpc.query(
          joinedPath,
          {
            ...queryKey[1]?.input,
            cursor: pageParam
          },
          {
            signal,
            ...trpcOptions
          }
        ),
        ...maybeToRefs(queryOptions)
      });
    }
    throw new Error(`Method '.${prop}()' not supported`);
  });
}
function createTRPCVueQueryClient({
  trpc,
  queryClient
}) {
  const client = createTRPCUntypedClient(trpc);
  const decoratedClient = createTRPCFlatProxy((key) => {
    return createVueQueryProxyDecoration(key.toString(), client, queryClient);
  });
  return decoratedClient;
}
export {
  createTRPCVueQueryClient
};
